import os
from xl.reader.json_io import JSONWriter, export_json
from xl.common.constant import (
    DEFAULT_ROW_COUNT,
    DEFAULT_COL_COUNT,
    SCHEMA_VERSION,
)
import io
import csv
import sys
from openpyxl.utils.inference import (
    cast_numeric,
    cast_percentage,
)


def get_index(title):
    return {
        'maxIndex': 1,
        'order': ['sh_1'],
        'sheets': {
            'sh_1': {'title': title, 'deleted': False}
        },
        'schemaVersion': SCHEMA_VERSION,
        'ver': 1,
        'gcVer': 1,
    }


def get_stylesheet():
    return {
        'styles': [], 'fonts': [], 'fills': [],
        'alignments': [], 'borders': [], 'numberFormats': [],
    }


def infer_value(value):
    if not value:
        return value
    return cast_numeric(value) or cast_percentage(value) or value


def write_cells(filename, encoding, writer):
    writer.write('"cells":{')
    max_row = 0
    max_col = 0
    cells_quota = 2000000
    cells_count = 0

    with io.open(filename, mode='r', encoding=encoding, errors='ignore', newline='') as csvfile:
        try:
            dialect = csv.Sniffer().sniff(csvfile.read(1024))
        except Exception as _:
            dialect = 'excel'
        csvfile.seek(0)
        reader = csv.reader(csvfile, dialect)

        row_count = 0
        for row in reader:
            r = {}
            for idx, cell in enumerate(row):
                val = infer_value(cell)
                if val:
                    r[idx] = {'v': val}
                    cells_count += 1
                    if cells_count > cells_quota:
                        sys.exit(255)
            if not r:
                max_row += 1
                continue

            if row_count != 0:
                writer.write(',')
            writer.write('"%d":' % max_row)
            writer.write_json(r)
            max_col = max(len(row), max_col)
            max_row += 1
            row_count += 1
    writer.write('},')
    return (max_row, max_col)


def import_csv(input_file, output_dir, encoding):
    export_json(get_index('Sheet1'), os.path.join(output_dir, 'index'))
    export_json(get_stylesheet(), os.path.join(output_dir, 'style'))

    filename = os.path.join(output_dir, 'sh_1')
    with JSONWriter(filename) as f:
        max_row, max_col = write_cells(input_file, encoding.lower(), f)
        f.write_dict({
            'rowCount': max(max_row + 1, DEFAULT_ROW_COUNT),
            'colCount': max(max_col + 1, DEFAULT_COL_COUNT),
            'rows': {},
            'cols': {},
            'global': None,
            'fixedRowTop': 0,
            'fixedColumnLeft': 0,
            'mergeCells': [],
            'filter': None,
            'validation': {},
            'cfs': [],
        })
        f.write_key_value('drawing', [], tail_comma=False)
