<?php
namespace PhpOffice\PhpPresentation\Writer\PowerPoint2007;

use PhpOffice\Common\Drawing as CommonDrawing;
use PhpOffice\Common\XMLWriter;
use PhpOffice\PhpPresentation\Shape\AbstractDrawing;
use PhpOffice\PhpPresentation\Shape\Chart as ShapeChart;
use PhpOffice\PhpPresentation\Shape\Comment;
use PhpOffice\PhpPresentation\Shape\RichText;
use PhpOffice\PhpPresentation\Shape\RichText\Paragraph;
use PhpOffice\PhpPresentation\Shape\Table as ShapeTable;
use PhpOffice\PhpPresentation\Slide;
use PhpOffice\PhpPresentation\Slide\SlideMaster;
use PhpOffice\PhpPresentation\Style\Alignment;
use PhpOffice\PhpPresentation\Style\Bullet;
use PhpOffice\PhpPresentation\Style\SchemeColor;
use PhpOffice\PhpPresentation\Slide\Background\Image;

class PptSlideMasters extends AbstractSlide
{
    /**
     * @return \PhpOffice\Common\Adapter\Zip\ZipInterface
     */
    public function render()
    {
        foreach ($this->oPresentation->getAllMasterSlides() as $oMasterSlide) {
            $this->resetRelationship();

            $masterIdx = $oMasterSlide->getRelsIndex();

            $this->addRelation($this->getRelationId(), 'theme', '../theme/theme' . $masterIdx . '.xml');

            $this->oZip->addFromString('ppt/slideMasters/slideMaster' . $masterIdx . '.xml', $this->writeSlideMaster($oMasterSlide));
            $this->oZip->addFromString('ppt/slideMasters/_rels/slideMaster' . $masterIdx . '.xml.rels', $this->writeRelationships());

            // Add background image slide
            $oBkgImage = $oMasterSlide->getBackground();
            if ($oBkgImage instanceof Image) {
                $this->oZip->addFromString('ppt/media/' . $oBkgImage->getIndexedFilename($masterIdx), file_get_contents($oBkgImage->getPath()));
            }
        }

        return $this->oZip;
    }

    /**
     * Write slide to XML format
     *
     * @param  \PhpOffice\PhpPresentation\Slide\SlideMaster $pSlide
     * @return string XML Output
     * @throws \Exception
     */
    public function writeSlideMaster(SlideMaster $pSlide)
    {
        // Create XML writer
        $objWriter = new XMLWriter(XMLWriter::STORAGE_MEMORY);
        // XML header
        $objWriter->startDocument('1.0', 'UTF-8', 'yes');
        // p:sldMaster
        $objWriter->startElement('p:sldMaster');
        $objWriter->writeAttribute('xmlns:a', 'http://schemas.openxmlformats.org/drawingml/2006/main');
        $objWriter->writeAttribute('xmlns:r', 'http://schemas.openxmlformats.org/officeDocument/2006/relationships');
        $objWriter->writeAttribute('xmlns:p', 'http://schemas.openxmlformats.org/presentationml/2006/main');
        // p:sldMaster\p:cSld
        $objWriter->startElement('p:cSld');
        // Background
        $this->writeSlideBackground($pSlide, $objWriter);
        // p:sldMaster\p:cSld\p:spTree
        $objWriter->startElement('p:spTree');
        // p:sldMaster\p:cSld\p:spTree\p:nvGrpSpPr
        $objWriter->startElement('p:nvGrpSpPr');
        // p:sldMaster\p:cSld\p:spTree\p:nvGrpSpPr\p:cNvPr
        $objWriter->startElement('p:cNvPr');
        $objWriter->writeAttribute('id', '1');
        $objWriter->writeAttribute('name', '');
        $objWriter->endElement();
        // p:sldMaster\p:cSld\p:spTree\p:nvGrpSpPr\p:cNvGrpSpPr
        $objWriter->writeElement('p:cNvGrpSpPr', null);
        // p:sldMaster\p:cSld\p:spTree\p:nvGrpSpPr\p:nvPr
        $objWriter->writeElement('p:nvPr', null);
        // p:sldMaster\p:cSld\p:spTree\p:nvGrpSpPr
        $objWriter->endElement();
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr
        $objWriter->startElement('p:grpSpPr');
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr\a:xfrm
        $objWriter->startElement('a:xfrm');
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr\a:xfrm\a:off
        $objWriter->startElement('a:off');
        $objWriter->writeAttribute('x', 0);
        $objWriter->writeAttribute('y', 0);
        $objWriter->endElement();
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr\a:xfrm\a:ext
        $objWriter->startElement('a:ext');
        $objWriter->writeAttribute('cx', 0);
        $objWriter->writeAttribute('cy', 0);
        $objWriter->endElement();
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr\a:xfrm\a:chOff
        $objWriter->startElement('a:chOff');
        $objWriter->writeAttribute('x', 0);
        $objWriter->writeAttribute('y', 0);
        $objWriter->endElement();
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr\a:xfrm\a:chExt
        $objWriter->startElement('a:chExt');
        $objWriter->writeAttribute('cx', 0);
        $objWriter->writeAttribute('cy', 0);
        $objWriter->endElement();
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr\a:xfrm\
        $objWriter->endElement();
        // p:sldMaster\p:cSld\p:spTree\p:grpSpPr\
        $objWriter->endElement();
        // Loop shapes
        $this->writeShapeCollection($objWriter, $pSlide->getShapeCollection());
        // p:sldMaster\p:cSld\p:spTree\
        $objWriter->endElement();
        // p:sldMaster\p:cSld\
        $objWriter->endElement();

        // p:sldMaster\p:clrMap
        $objWriter->startElement('p:clrMap');
        foreach ($pSlide->colorMap->getMapping() as $n => $v) {
            $objWriter->writeAttribute($n, $v);
        }
        $objWriter->endElement();
        // p:sldMaster\p:clrMap\

        // p:sldMaster\p:sldLayoutIdLst
        $objWriter->startElement('p:sldLayoutIdLst');
        foreach ($pSlide->getAllSlideLayouts() as $layout) {
            $rId = $this->getRelationId();
            $this->addRelation($rId, 'layout', '../slideLayouts/slideLayout' . $layout->layoutNr . '.xml');

            /* @var $layout Slide\SlideLayout */
            $objWriter->startElement('p:sldLayoutId');
            $objWriter->writeAttribute('id', $layout->layoutId);
            $objWriter->writeAttribute('r:id', $rId);
            $objWriter->endElement();
        }
        $objWriter->endElement();
        // p:sldMaster\p:sldLayoutIdLst\

        // p:sldMaster\p:txStyles
        $objWriter->startElement('p:txStyles');
        foreach (array(
                     'p:titleStyle' => $pSlide->getTextStyles()->getTitleStyle(),
                     'p:bodyStyle' => $pSlide->getTextStyles()->getBodyStyle(),
                     'p:otherStyle' => $pSlide->getTextStyles()->getOtherStyle()
                 ) as $startElement => $stylesArray) {
            // titleStyle
            $objWriter->startElement($startElement);

            foreach ($stylesArray as $lvl => $oParagraph) {
                /** @var RichText\Paragraph $oParagraph */
                $elementName = 'a:lvl' . (intval($lvl) + 1)  . 'pPr';
                $objWriter->startElement($elementName);
                if ('' !== $oParagraph->getAlignment()->getHorizontal()) {
                    $objWriter->writeAttribute('algn', $oParagraph->getAlignment()->getHorizontal());
                } else {
                    $objWriter->writeAttribute('algn', 'l');
                }

                //FIXME
                $objWriter->writeAttribute('defTabSz', '914400');
                if ('p:bodyStyle' === $startElement) {
                    $marL = (Alignment::MARGIN_NONE !== $oParagraph->getAlignment()->getMarginLeft()) ? 
                        CommonDrawing::pixelsToEmu($oParagraph->getAlignment()->getMarginLeft()) :
                        (intval($lvl) + 1) * 285750;
                    $objWriter->writeAttribute('marL', $marL);
                    $objWriter->writeAttributeIf(
                        Alignment::MARGIN_NONE !== $oParagraph->getAlignment()->getMarginRight(),
                        'marR',
                        CommonDrawing::pixelsToEmu($oParagraph->getAlignment()->getMarginRight())
                    );
                    $indent = (Alignment::INDENT_NONE !== $oParagraph->getAlignment()->getIndent()) ?
                        CommonDrawing::pixelsToEmu($oParagraph->getAlignment()->getIndent()) :
                        -285750;
                    $objWriter->writeAttribute('indent', $indent);
                } else {
                    if (Alignment::MARGIN_NONE !== $oParagraph->getAlignment()->getMarginLeft()) {
                        $objWriter->writeAttribute('marL', CommonDrawing::pixelsToEmu($oParagraph->getAlignment()->getMarginLeft()));
                    }
                }

                if (Paragraph::LINE_SPACE_NONE !== $oParagraph->getLineSpacing()) {
                    $objWriter->startElement('a:lnSpc');
                    $objWriter->startElement('a:spcPct');
                    $objWriter->writeAttribute('val', $oParagraph->getLineSpacing());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                if (Paragraph::SPACE_BEF_AFT_NONE !== $oParagraph->getSpaceBefore()) {
                    $objWriter->startElement('a:spcBef');
                    $objWriter->startElement('a:spcPts');
                    $objWriter->writeAttribute('val', $oParagraph->getSpaceBefore());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                if (Paragraph::SPACE_BEF_AFT_NONE !== $oParagraph->getSpaceAfter()) {
                    $objWriter->startElement('a:spcAft');
                    $objWriter->startElement('a:spcPts');
                    $objWriter->writeAttribute('val', $oParagraph->getSpaceAfter());
                    $objWriter->endElement();
                    $objWriter->endElement();
                }

                $oFont = $oParagraph->getFont();
                $bulletType = $oParagraph->getBulletStyle()->getBulletType();
                if (Bullet::TYPE_NONE !== $bulletType && Bullet::TYPE_NOTSET !== $bulletType) {
                    if (Bullet::TYPE_BULLET === $bulletType) {
                        $objWriter->startElement('a:buFont');
                        $objWriter->writeAttribute('typeface', $oFont->getName());
                        $objWriter->endElement();
                    } else {
                        $objWriter->startElement('a:buAutoNum');
                        $objWriter->writeAttribute('type', 'arabicPeriod');
                        $objWriter->endElement();
                    }
                }

                $objWriter->startElement('a:defRPr');
                $this->writeParagraphProperty($objWriter, $oFont);
                $objWriter->writeAttribute('kern', '1200');
                $objWriter->endElement();

                $objWriter->endElement();
            }
            $objWriter->writeElement('a:extLst');
            $objWriter->endElement();
        }
        $objWriter->endElement();
        // p:sldMaster\p:txStyles\

        $this->writeSlideTransition($objWriter, $pSlide->getTransition());
        $this->writeSlideAnimations($objWriter, $pSlide->getAnimations());

        // p:sldMaster\
        $objWriter->endElement();

        return $objWriter->getData();
    }
}
