<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\Style\Border;

class FloatingObject extends Node
{
    public $id;
    protected $slide;
    protected $container;
    protected $phpShape;

    public function __construct($slide, $container)
    {
        parent::__construct();
        $this->id = Utils::genSynoId();
        $this->slide = $slide;
        $this->container = $container;
    }

    public function setData($phpShape)
    {
        $this->phpShape = $phpShape;

        $this->setAttr('id', $this->id);
        $this->setTransform();
        $this->setContent();
        $this->setSynolink();
    }

    protected function setTransform()
    {
        $transform = new \stdClass();
        $deg = $this->phpShape->getRotation();
        $height = $this->phpShape->getHeight();
        $width = $this->phpShape->getWidth();
        $x = $this->phpShape->getOffsetX();
        $y = $this->phpShape->getOffsetY();
        $flipH = $this->phpShape->getFlipH();
        $flipV = $this->phpShape->getFlipV();

        if (is_null($deg) && is_null($height) && is_null($width) && is_null($x) && is_null($y)) {
            return;
        }

        if (!is_null($deg)) {
            $transform->deg = $deg;
        }

        if (!is_null($height) && !is_null($width)) {
            //FIXME
            if ($this instanceof LineBox) {
                if (1 > abs($width)) {
                    $width = 1;
                }
                if (1 > abs($height)) {
                    $height = 1;
                }
            }

            if ($this->container instanceof ObjectGroup) {
                $width *= $this->container->getChildScaleX();
                $height *= $this->container->getChildScaleY();
            }

            if ($flipH) {
                $width = $width * -1;
            }
            if ($flipV) {
                $height = $height * -1;
            }

            $transform->height = $height;
            $transform->width = $width;
        }

        if (!is_null($x) && !is_null($y)) {
            if ($this->container instanceof ObjectGroup) {
                $x -= $this->container->getChildOffsetX();
                $y -= $this->container->getChildOffsetY();

                $x *= $this->container->getChildScaleX();
                $y *= $this->container->getChildScaleY();
            }
            if ($flipH) {
                $x += abs($width);
            }
            if ($flipV) {
                $y += abs($height);
            }
            if (!is_null($deg) && (0 != $transform->deg || 180 != $transform->deg)) {
                $rad = deg2rad($transform->deg);
                $centerX = $x + $transform->width / 2;
                $centerY = $y + $transform->height / 2;
                $transform->left = ($x - $centerX) * cos($rad) + $centerX - ($y - $centerY) * sin($rad);
                $transform->top = ($y - $centerY) * cos($rad) + $centerY + ($x - $centerX) * sin($rad);
            } else {
                $transform->left = $x;
                $transform->top = $y;
            }
        }
        $transform->skewY = 0;

        $this->setAttr('transform', $transform);
    }

    private function setSynolink()
    {
        $hyperlink = $this->phpShape->getHyperlink();
        if ('' !== $hyperlink->getAction() || '' !== $hyperlink->getUrl()) {
            if (!$hyperlink->isInternal()) {
                $href = $hyperlink->getUrl();
            } else if (!$hyperlink->isRelative()) {
                $slideNum = $hyperlink->getSlideNumber();
                $href = '#' . $this->slide->getPresentation()->getSlideId($slideNum - 1);
            } else {
                switch ($hyperlink->getRelativePos()) {
                    case 'firstslide':
                        $slideId = $this->slide->getPresentation()->getSlideId(0);
                        break;
                    case 'lastslide':
                        $slideId = $this->slide->getPresentation()->getLastSlideId();
                        break;
                    case 'nextslide':
                        $slideId = $this->slide->getPresentation()->getSlideId($this->slide->getIdx() + 1);
                        break;
                    case 'previousslide':
                        $slideId = $this->slide->getPresentation()->getSlideId($this->slide->getIdx() - 1);
                        break;
                    default:
                        $slideId = '';
                        break;
                }
                $href = '#' . $slideId;
            }
            $this->setAttr('synolink', $href);
        }
    }

    protected function setBorder($border, $strokeAttrs)
    {
        if (!isset($border)) {
            return;
        }
        $strokeWidth = 'stroke-width';
        $strokeDasharray = 'stroke-dasharray';
        $strokeOpacity = 'stroke-opacity';

        if (Border::LINE_NONE === $border->getLineStyle() || Border::LINE_NOTSET === $border->getLineStyle()) {
            $strokeAttrs->stroke = false;
        }

        $strokeAttrs->stroke_config->$strokeWidth = $border->getLineWidth();
        $dashStyle = strtolower($border->getDashStyle());
        if (false !== strpos($dashStyle, 'dash')) {
            $strokeAttrs->stroke_config->$strokeDasharray = '6,6';
        } else if (false !== strpos($dashStyle, 'dot')) {
            $strokeAttrs->stroke_config->$strokeDasharray = '2,2';
        } else {
            $strokeAttrs->stroke_config->$strokeDasharray = '';
        }
        $strokeAttrs->stroke_config->stroke = '#' . $this->slide->getColor($border->getColor());
        $strokeAttrs->stroke_config->$strokeOpacity = $this->slide->getOpacity($border->getColor());
    }
}

?>
