<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\Style\Alignment;
use PhpOffice\PhpPresentation\Style\Bullet;

class Note extends Node
{
    public $type = 'note';
    protected $slide;

    public function setSlide($slide)
    {
        $this->slide = $slide;
    }

    public function setData($phpNote)
    {
        $phpShapes = $phpNote->getShapeCollection();
        foreach ($phpShapes as $phpShape) {
            if (!$this->isSupportedShape($phpShape)) {
                continue;
            }
            $phpParagraphs = $phpShape->getParagraphs();

            $preType = Bullet::TYPE_NOTSET;
            $preSubType = '';
            $levelIndentMapping = array();
            foreach ($phpParagraphs as $phpParagraph) {
                $noText = false;
                if (0 === count($phpParagraph->getRichTextElements())) {
                    $noText = true;
                }
                $paragraph = new Paragraph();
                $paragraph->setSlide($this->slide);

                $alignment = $phpParagraph->getAlignment();
                $level = $alignment->getLevel() + 1;
                $bulletType = Bullet::TYPE_NOTSET;

                $marginLeft = (Alignment::MARGIN_NONE === $alignment->getMarginLeft()) ? 0 : $alignment->getMarginLeft();
                $indent = (Alignment::INDENT_NONE === $alignment->getIndent()) ? 0 : $alignment->getIndent();
                $paddingLeft = $marginLeft + $indent;

                $listStyle = $phpShape->getListStyle()->getStyleAtLvl($level);
                if (isset($listStyle)) {
                    $bulletType = $listStyle->getBulletStyle()->getBulletType();
                }
                $paragraph->setListStyle($listStyle);
                $paragraph->setData($phpParagraph, false);

                if (Bullet::TYPE_NOTSET !== $phpParagraph->getBulletStyle()->getBulletType()) {
                    $bulletType = $phpParagraph->getBulletStyle()->getBulletType();
                }

                if (Bullet::TYPE_NOTSET === $bulletType) {
                    $bulletType = Bullet::TYPE_NONE;
                }

                if (Bullet::TYPE_NONE !== $bulletType && !$noText) {
                    if (Bullet::TYPE_NOTSET === $preType || Bullet::TYPE_NONE === $preType || $level < $preLevel - 1) {
                        $preLevel = -1;
                        $container = $this;
                        $listStack = array();
                    }

                    if (1 < $level && -1 === $preLevel) {
                        for ($i = 1; $i < $level; $i++) {
                            $list = ('numeric' === $bulletType) ? new OrderedList() : new BulletList();
                            $container->addContent($list);
                            $listItem = new ListItem();
                            $list->addContent($listItem);
                            $container = $listItem;
                        }
                        $preLevel = $level - 1;
                    }

                    if (1 === $level && !isset($paddingBase)) {
                        $paddingBase = $paddingLeft;
                    }

                    if (!isset($levelIndentMapping[$level])) {
                        $levelIndentMapping[$level] = $paddingLeft;
                    } else if ($levelIndentMapping[$level] != $paddingLeft && 0 < $paddingLeft - $paddingBase) {
                        $paragraph->setAttr('paddingLeft', ($paddingLeft - $paddingBase) . 'px');
                    }

                    $subType = (Bullet::TYPE_NUMERIC === $bulletType) ? $phpParagraph->getBulletStyle()->getBulletNumericStyle() : '';

                    if ($preLevel < $level) {
                        if (-1 !== $preLevel) {
                            array_push($listStack, $list);
                        }
                        $list = ('numeric' === $bulletType) ? new OrderedList() : new BulletList();
                        $container->addContent($list);
                    } else if ($preLevel > $level) {
                        $list = array_pop($listStack);
                    } else if (Bullet::TYPE_NUMERIC === $bulletType && $subType != $preSubType) {
                        if (1 === $level) {
                            $list = new OrderedList();
                            $this->addContent($list);
                        } else {
                            $list = array_pop($listStack);
                        }
                    }
                    $preLevel = $level;
                    $listItem = new ListItem();

                    $list->addContent($listItem);
                    $container = $listItem;
                } else if (Bullet::TYPE_NONE == $bulletType) {
                    if (1 < $level) {
                        $paragraph->setAttr('paddingLeft', ($level * 30) . 'px');
                    } else if (Alignment::INDENT_NONE != $phpParagraph->getAlignment()->getIndent() && 0 < $phpParagraph->getAlignment()->getIndent()) {
                        $paragraph->setAttr('paddingLeft', Utils::getIndent($phpParagraph->getAlignment()->getIndent(), $levelIndentMapping));
                    }
                    $preLevel = -1;
                    $container = $this;
                }
                if (!isset($container)) {
                    $container = $this;
                }
                $preType = $bulletType;
                $preSubType = $subType;

                $container->addContent($paragraph);
            }

            if (0 === count($phpParagraphs)) {
                $paragraph = new Paragraph();
                $this->addContent($paragraph);
            }
        }

        if (0 === count($phpShapes)) {
            $paragraph = new Paragraph();
            $this->addContent($paragraph);
        }
    }

    protected function isSupportedShape($phpShape)
    {
        if (!$phpShape->isPlaceHolder()) {
            return false;
        }
        $phpHolder = $phpShape->getPlaceHolder();
        if (!isset($phpHolder) || 'body' !== $phpHolder->getType()) {
            return false;
        }
        return true;
    }
}

?>
