<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element;

/**
 * TextRun element HTML writer
 *
 * @since 0.10.0
 */
class TextRun extends Text
{
    /**
     * Write text run
     *
     * @return string
     */
    public function write()
    {
        $pStyle = $this->element->getParagraphStyle();
        $content = '';

        // writing hr if top border exists
        if (method_exists($pStyle, 'getBorderTopSize') && !is_null($pStyle->getBorderTopSize(true))) {
            $content .= '<div><hr></div>';
        }
        $content .= $this->writeOpening();
        $writer = new Container($this->parentWriter, $this->element);
        $content .= $writer->write();
        $content .= $this->writeClosing();

        // writing hr if bottom border exists
        if (method_exists($pStyle, 'getBorderBottomSize') && !is_null($pStyle->getBorderBottomSize(true))) {
            $content .= '<div><hr></div>';
        }

        return $content;
    }
}
