<?php
namespace PhpOffice\PhpWord\Writer\Word2007\Element;

class Inline extends AbstractElement
{
    public function write()
    {
        // Do nothing
    }

    protected function writeCommentRangeStart($xmlWriter, $element)
    {
        global $comment;
        if (!$element instanceof \PhpOffice\PhpWord\Element\Inline) {
            return;
        }
        if ( !(is_array($comment) && method_exists($element, 'getComments')) ) {
            return;
        }
        foreach ($element->getComments() as $id) {
            if ( !array_key_exists($id, $comment) ) {
                continue;
            }
            if ( $comment[$id]['@begin'] !== $element ) {
                continue;
            }
            $id = $comment[$id]['@id'];
            $xmlWriter->startElement('w:commentRangeStart');
            $xmlWriter->writeAttribute('w:id', $id);
            $xmlWriter->endElement(); // w:commentRangeStart
        }
    }

    protected function writeCommentRangeEnd($xmlWriter, $element)
    {
        global $comment;
        if (!$element instanceof \PhpOffice\PhpWord\Element\Inline) {
            return;
        }
        if ( !(is_array($comment) && method_exists($element, 'getComments')) ) {
            return;
        }
        foreach ($element->getComments() as $id) {
            if ( !array_key_exists($id, $comment) ) {
                continue;
            }
            if ( $comment[$id]['@end'] !== $element ) {
                continue;
            }
            $id = $comment[$id]['@id'];
            $xmlWriter->startElement('w:commentRangeEnd');
            $xmlWriter->writeAttribute('w:id', $id);
            $xmlWriter->endElement(); // w:commentRangeEnd
            $xmlWriter->startElement('w:r');
            $xmlWriter->startElement('w:commentReference');
            $xmlWriter->writeAttribute('w:id', $id);
            $xmlWriter->endElement(); // w:commentReference
            $xmlWriter->endElement(); // w:r
        }
    }

    protected function writeRevisionOpenTag($xmlWriter, $element)
    {
        global $comment;
        if (!$element instanceof \PhpOffice\PhpWord\Element\Inline) {
            return;
        }
        $revision = $element->getRevision();
        if (!is_array($revision) || !is_array($comment)) {
            return;
        }
        if ( !empty($comment[$revision['id']]['replies']) ) {
            foreach ( $comment[$revision['id']]['replies'] as $c ) {
                if ( !array_key_exists($c['id'], $comment) ) {
                    continue;
                }
                $id = $comment[$c['id']]['@id'];
                $xmlWriter->startElement('w:commentRangeStart');
                $xmlWriter->writeAttribute('w:id', $id);
                $xmlWriter->endElement(); // w:commentRangeStart
            }
        }
        $xmlWriter->startElement("w:{$revision['op']}");
        $xmlWriter->writeAttribute('w:id', $revision['@id']);
        $xmlWriter->writeAttribute('w:author', $revision['author']);
        $xmlWriter->writeAttribute('w:date', $revision['date']);
    }

    protected function writeRevisionCloseTag($xmlWriter, $element)
    {
        global $comment;
        if (!$element instanceof \PhpOffice\PhpWord\Element\Inline) {
            return;
        }
        $revision = $element->getRevision();
        if (!is_array($revision) || !is_array($comment)) {
            return;
        }
        $xmlWriter->endElement(); // w:del or w:ins

        if ( !empty($comment[$revision['id']]['replies']) ) {
            foreach ( $comment[$revision['id']]['replies'] as $c ) {
                if ( !array_key_exists($c['id'], $comment) ) {
                    continue;
                }
                $id = $comment[$c['id']]['@id'];
                $xmlWriter->startElement('w:commentRangeEnd');
                $xmlWriter->writeAttribute('w:id', $id);
                $xmlWriter->endElement(); // w:commentRangeEnd
                $xmlWriter->startElement('w:r');
                $xmlWriter->startElement('w:commentReference');
                $xmlWriter->writeAttribute('w:id', $id);
                $xmlWriter->endElement(); // w:commentReference
                $xmlWriter->endElement(); // w:r
            }
        }
    }
}
